from asyncio import AbstractEventLoop
from ssl import SSLContext
from tempfile import _DirT
from typing import Any, Callable, Dict, Optional, Tuple, Type, TypeVar, Union, overload
from typing_extensions import TypedDict

from . import connect_utils
from .connection import Connection, _PasswordType
from .types import ServerVersion

class _ConnectionSpec(TypedDict):
    host: str
    port: str

def platform_exe(name: str) -> str: ...
def find_available_port() -> Optional[int]: ...

class ClusterError(Exception): ...

class Cluster:
    def __init__(
        self, data_dir: str, *, pg_config_path: Optional[str] = ...
    ) -> None: ...
    def get_pg_version(self) -> ServerVersion: ...
    def is_managed(self) -> bool: ...
    def get_data_dir(self) -> str: ...
    def get_status(self) -> str: ...
    async def connect(
        self,
        loop: Optional[AbstractEventLoop] = ...,
        **kwargs: Any,
    ) -> Connection[Any]: ...
    def init(self, **settings: Any) -> str: ...
    def start(
        self, wait: int = ..., *, server_settings: Dict[str, str] = ..., **opts: Any
    ) -> None: ...
    def reload(self) -> None: ...
    def stop(self, wait: int = ...) -> None: ...
    def destroy(self) -> None: ...
    def get_connection_spec(self) -> _ConnectionSpec: ...
    def override_connection_spec(self, **kwargs: Any) -> None: ...
    def reset_wal(
        self, *, oid: Optional[int] = ..., xid: Optional[int] = ...
    ) -> None: ...
    def reset_hba(self) -> None: ...
    def add_hba_entry(
        self,
        *,
        type: str = ...,
        database: str,
        user: str,
        address: Optional[str] = ...,
        auth_method: str,
        auth_options: Optional[Dict[str, str]] = ...,
    ) -> None: ...
    def trust_local_connections(self) -> None: ...
    def trust_local_replication_by(self, user: str) -> None: ...

class TempCluster(Cluster):
    def __init__(
        self,
        *,
        data_dir_suffix: Optional[str] = ...,
        data_dir_prefix: Optional[str] = ...,
        data_dir_parent: Optional[_DirT[str]] = ...,
        pg_config_path: Optional[str] = ...,
    ) -> None: ...

class HotStandbyCluster(TempCluster):
    def __init__(
        self,
        *,
        master: _ConnectionSpec,
        replication_user: str,
        data_dir_suffix: Optional[str] = ...,
        data_dir_prefix: Optional[str] = ...,
        data_dir_parent: Optional[_DirT[str]] = ...,
        pg_config_path: Optional[str] = ...,
    ) -> None: ...
    def init(self, **settings: str) -> str: ...

class RunningCluster(Cluster):
    conn_spec: _ConnectionSpec = ...
    def __init__(self, **kwargs: str) -> None: ...
    def is_managed(self) -> bool: ...
    def get_connection_spec(self) -> _ConnectionSpec: ...
    def get_status(self) -> str: ...
    def init(self, **settings: str) -> str: ...
    def start(self, wait: int = ..., **settings: Any) -> None: ...
    def stop(self, wait: int = ...) -> None: ...
    def destroy(self) -> None: ...
    def reset_hba(self) -> None: ...
    def add_hba_entry(
        self,
        *,
        type: str = ...,
        database: str,
        user: str,
        address: Optional[str] = ...,
        auth_method: str,
        auth_options: Optional[Dict[str, str]] = ...,
    ) -> None: ...



from .protocol import BUILTIN_TYPE_NAME_MAP
from .protocol import NO_TIMEOUT as NO_TIMEOUT
from .protocol import Protocol as Protocol
from .protocol import Record as Record

__all__ = ('NO_TIMEOUT', 'Protocol', 'Record', 'BUILTIN_TYPE_NAME_MAP')



from asyncio import AbstractEventLoop, Protocol, WriteTransport
from enum import IntEnum
from ssl import SSLContext
from typing import Any, Dict, List, NamedTuple, Optional, Tuple, Type, TypeVar, Union
from typing_extensions import Final, Literal

_Connection = TypeVar('_Connection')
_Protocol = TypeVar('_Protocol', bound=Protocol)
_SSLMode = TypeVar('_SSLMode', bound=SSLMode)

_TPTupleType = Tuple[WriteTransport, _Protocol]
_AddrType = Union[Tuple[str, int], str]
_HostType = Union[List[str], str]
_PortType = Union[List[int], int]
_ParsedSSLType = Union[SSLContext, Literal[False]]
_SSLStringValues = Literal[
    'disable', 'prefer', 'allow', 'require', 'verify-ca', 'verify-full'
]
_SSLType = Union[_ParsedSSLType, _SSLStringValues, bool]

PGPASSFILE: Final[str]

class SSLMode(IntEnum):
    disable: int
    allow: int
    prefer: int
    require: int
    verify_ca: int
    verify_full: int
    @classmethod
    def parse(cls: Type[_SSLMode], sslmode: Union[str, _SSLMode]) -> _SSLMode: ...

class _ConnectionParameters(NamedTuple):
    user: str
    password: Optional[str]
    database: str
    ssl: Optional[_ParsedSSLType]
    sslmode: Optional[SSLMode]
    connect_timeout: Optional[float]
    server_settings: Optional[Dict[str, str]]

class _ClientConfiguration(NamedTuple):
    command_timeout: Optional[float]
    statement_cache_size: int
    max_cached_statement_lifetime: int
    max_cacheable_statement_size: int

class TLSUpgradeProto(Protocol):
    on_data: Any = ...
    host: Any = ...
    port: Any = ...
    ssl_context: Any = ...
    ssl_is_advisory: Any = ...
    def __init__(
        self,
        loop: Optional[AbstractEventLoop],
        host: str,
        port: int,
        ssl_context: SSLContext,
        ssl_is_advisory: Optional[bool],
    ) -> None: ...
    def data_received(self, data: bytes) -> None: ...
    def connection_lost(self, exc: Optional[Exception]) -> None: ...



from typing import Any, Callable, TypeVar

from .connection import Connection

_Callable = TypeVar('_Callable', bound=Callable[..., Any])

def guarded(meth: _Callable) -> _Callable: ...

class ConnectionResource:
    def __init__(self, connection: Connection[Any]) -> None: ...



from typing import Any, Tuple
from typing_extensions import Final, TypedDict

from .protocol import Record

INTRO_LOOKUP_TYPES: Final[str]
TYPE_BY_NAME: Final[str]
TYPE_BY_OID: Final[str]
SCALAR_TYPE_KINDS: Final[Tuple[bytes, bytes, bytes]]

def is_scalar_type(typeinfo: Record) -> bool: ...
def is_domain_type(typeinfo: Record) -> bool: ...



import unittest
from typing import Any, Callable, Iterator, Optional, TypeVar

from . import fuzzer as fuzzer

_F = TypeVar('_F', bound=Callable[..., Any])

def silence_asyncio_long_exec_warning() -> Iterator[None]: ...
def with_timeout(timeout: Any) -> Callable[[_F], _F]: ...

class TestCaseMeta(type):
    TEST_TIMEOUT: Any = ...
    def __new__(mcls: Any, name: Any, bases: Any, ns: Any) -> Any: ...

class TestCase(unittest.TestCase, metaclass=TestCaseMeta):
    @classmethod
    def setUpClass(cls) -> None: ...
    @classmethod
    def tearDownClass(cls) -> None: ...
    def setUp(self) -> None: ...
    def tearDown(self) -> None: ...
    def assertRunUnder(self, delta: Any) -> None: ...
    def assertLoopErrorHandlerCalled(self, msg_re: str) -> Any: ...
    def loop_exception_handler(self, loop: Any, context: Any) -> None: ...

def create_pool(
    dsn: Optional[Any] = ...,
    *,
    min_size: int = ...,
    max_size: int = ...,
    max_queries: int = ...,
    max_inactive_connection_lifetime: float = ...,
    setup: Optional[Any] = ...,
    init: Optional[Any] = ...,
    loop: Optional[Any] = ...,
    pool_class: Any = ...,
    connection_class: Any = ...,
    record_class: Any = ...,
    **connect_kwargs: Any,
) -> Any: ...

class ClusterTestCase(TestCase):
    @classmethod
    def get_server_settings(cls) -> Any: ...
    @classmethod
    def new_cluster(
        cls, ClusterCls: Any, *, cluster_kwargs: Any = ..., initdb_options: Any = ...
    ) -> Any: ...
    @classmethod
    def start_cluster(cls, cluster: Any, *, server_settings: Any = ...) -> None: ...
    @classmethod
    def setup_cluster(cls) -> None: ...
    @classmethod
    def setUpClass(cls) -> None: ...
    @classmethod
    def tearDownClass(cls) -> None: ...
    @classmethod
    def get_connection_spec(cls, kwargs: Any = ...) -> Any: ...
    @classmethod
    def connect(cls, **kwargs: Any) -> Any: ...
    def setUp(self) -> None: ...
    def tearDown(self) -> None: ...
    def create_pool(
        self, pool_class: Any = ..., connection_class: Any = ..., **kwargs: Any
    ) -> Any: ...

class ProxiedClusterTestCase(ClusterTestCase):
    @classmethod
    def get_server_settings(cls) -> Any: ...
    @classmethod
    def get_proxy_settings(cls) -> Any: ...
    @classmethod
    def setUpClass(cls) -> None: ...
    @classmethod
    def tearDownClass(cls) -> None: ...
    @classmethod
    def get_connection_spec(cls, kwargs: Any) -> Any: ...  # type: ignore[override] # noqa
    def tearDown(self) -> None: ...

def with_connection_options(**options: Any) -> Any: ...

class ConnectedTestCase(ClusterTestCase):
    con: Any = ...
    server_version: Any = ...
    def setUp(self) -> None: ...
    def tearDown(self) -> None: ...



from . import _base
from ._base import FatalPostgresError as FatalPostgresError
from ._base import InterfaceError as InterfaceError
from ._base import InterfaceWarning as InterfaceWarning
from ._base import InternalClientError as InternalClientError
from ._base import OutdatedSchemaCacheError as OutdatedSchemaCacheError
from ._base import PostgresError as PostgresError
from ._base import PostgresLogMessage as PostgresLogMessage
from ._base import ProtocolError as ProtocolError
from ._base import UnknownPostgresError as UnknownPostgresError
from ._base import UnsupportedClientFeatureError as UnsupportedClientFeatureError

class PostgresWarning(_base.PostgresLogMessage, Warning): ...
class DynamicResultSetsReturned(PostgresWarning): ...
class ImplicitZeroBitPadding(PostgresWarning): ...
class NullValueEliminatedInSetFunction(PostgresWarning): ...
class PrivilegeNotGranted(PostgresWarning): ...
class PrivilegeNotRevoked(PostgresWarning): ...
class StringDataRightTruncation(PostgresWarning): ...
class DeprecatedFeature(PostgresWarning): ...
class NoData(PostgresWarning): ...
class NoAdditionalDynamicResultSetsReturned(NoData): ...
class SQLStatementNotYetCompleteError(_base.PostgresError): ...
class PostgresConnectionError(_base.PostgresError): ...
class ConnectionDoesNotExistError(PostgresConnectionError): ...
class ConnectionFailureError(PostgresConnectionError): ...
class ClientCannotConnectError(PostgresConnectionError): ...
class ConnectionRejectionError(PostgresConnectionError): ...
class TransactionResolutionUnknownError(PostgresConnectionError): ...
class ProtocolViolationError(PostgresConnectionError): ...
class TriggeredActionError(_base.PostgresError): ...
class FeatureNotSupportedError(_base.PostgresError): ...
class InvalidCachedStatementError(FeatureNotSupportedError): ...
class InvalidTransactionInitiationError(_base.PostgresError): ...
class LocatorError(_base.PostgresError): ...
class InvalidLocatorSpecificationError(LocatorError): ...
class InvalidGrantorError(_base.PostgresError): ...
class InvalidGrantOperationError(InvalidGrantorError): ...
class InvalidRoleSpecificationError(_base.PostgresError): ...
class DiagnosticsError(_base.PostgresError): ...
class StackedDiagnosticsAccessedWithoutActiveHandlerError(DiagnosticsError): ...
class CaseNotFoundError(_base.PostgresError): ...
class CardinalityViolationError(_base.PostgresError): ...
class DataError(_base.PostgresError): ...
class ArraySubscriptError(DataError): ...
class CharacterNotInRepertoireError(DataError): ...
class DatetimeFieldOverflowError(DataError): ...
class DivisionByZeroError(DataError): ...
class ErrorInAssignmentError(DataError): ...
class EscapeCharacterConflictError(DataError): ...
class IndicatorOverflowError(DataError): ...
class IntervalFieldOverflowError(DataError): ...
class InvalidArgumentForLogarithmError(DataError): ...
class InvalidArgumentForNtileFunctionError(DataError): ...
class InvalidArgumentForNthValueFunctionError(DataError): ...
class InvalidArgumentForPowerFunctionError(DataError): ...
class InvalidArgumentForWidthBucketFunctionError(DataError): ...
class InvalidCharacterValueForCastError(DataError): ...
class InvalidDatetimeFormatError(DataError): ...
class InvalidEscapeCharacterError(DataError): ...
class InvalidEscapeOctetError(DataError): ...
class InvalidEscapeSequenceError(DataError): ...
class NonstandardUseOfEscapeCharacterError(DataError): ...
class InvalidIndicatorParameterValueError(DataError): ...
class InvalidParameterValueError(DataError): ...
class InvalidPrecedingOrFollowingSizeError(DataError): ...
class InvalidRegularExpressionError(DataError): ...
class InvalidRowCountInLimitClauseError(DataError): ...
class InvalidRowCountInResultOffsetClauseError(DataError): ...
class InvalidTablesampleArgumentError(DataError): ...
class InvalidTablesampleRepeatError(DataError): ...
class InvalidTimeZoneDisplacementValueError(DataError): ...
class InvalidUseOfEscapeCharacterError(DataError): ...
class MostSpecificTypeMismatchError(DataError): ...
class NullValueNotAllowedError(DataError): ...
class NullValueNoIndicatorParameterError(DataError): ...
class NumericValueOutOfRangeError(DataError): ...
class SequenceGeneratorLimitExceededError(DataError): ...
class StringDataLengthMismatchError(DataError): ...
class StringDataRightTruncationError(DataError): ...
class SubstringError(DataError): ...
class TrimError(DataError): ...
class UnterminatedCStringError(DataError): ...
class ZeroLengthCharacterStringError(DataError): ...
class PostgresFloatingPointError(DataError): ...
class InvalidTextRepresentationError(DataError): ...
class InvalidBinaryRepresentationError(DataError): ...
class BadCopyFileFormatError(DataError): ...
class UntranslatableCharacterError(DataError): ...
class NotAnXmlDocumentError(DataError): ...
class InvalidXmlDocumentError(DataError): ...
class InvalidXmlContentError(DataError): ...
class InvalidXmlCommentError(DataError): ...
class InvalidXmlProcessingInstructionError(DataError): ...
class DuplicateJsonObjectKeyValueError(DataError): ...
class InvalidJsonTextError(DataError): ...
class InvalidSQLJsonSubscriptError(DataError): ...
class MoreThanOneSQLJsonItemError(DataError): ...
class NoSQLJsonItemError(DataError): ...
class NonNumericSQLJsonItemError(DataError): ...
class NonUniqueKeysInAJsonObjectError(DataError): ...
class SingletonSQLJsonItemRequiredError(DataError): ...
class SQLJsonArrayNotFoundError(DataError): ...
class SQLJsonMemberNotFoundError(DataError): ...
class SQLJsonNumberNotFoundError(DataError): ...
class SQLJsonObjectNotFoundError(DataError): ...
class TooManyJsonArrayElementsError(DataError): ...
class TooManyJsonObjectMembersError(DataError): ...
class SQLJsonScalarRequiredError(DataError): ...
class IntegrityConstraintViolationError(_base.PostgresError): ...
class RestrictViolationError(IntegrityConstraintViolationError): ...
class NotNullViolationError(IntegrityConstraintViolationError): ...
class ForeignKeyViolationError(IntegrityConstraintViolationError): ...
class UniqueViolationError(IntegrityConstraintViolationError): ...
class CheckViolationError(IntegrityConstraintViolationError): ...
class ExclusionViolationError(IntegrityConstraintViolationError): ...
class InvalidCursorStateError(_base.PostgresError): ...
class InvalidTransactionStateError(_base.PostgresError): ...
class ActiveSQLTransactionError(InvalidTransactionStateError): ...
class BranchTransactionAlreadyActiveError(InvalidTransactionStateError): ...
class HeldCursorRequiresSameIsolationLevelError(InvalidTransactionStateError): ...
class InappropriateAccessModeForBranchTransactionError(
    InvalidTransactionStateError
): ...
class InappropriateIsolationLevelForBranchTransactionError(
    InvalidTransactionStateError
): ...
class NoActiveSQLTransactionForBranchTransactionError(InvalidTransactionStateError): ...
class ReadOnlySQLTransactionError(InvalidTransactionStateError): ...
class SchemaAndDataStatementMixingNotSupportedError(InvalidTransactionStateError): ...
class NoActiveSQLTransactionError(InvalidTransactionStateError): ...
class InFailedSQLTransactionError(InvalidTransactionStateError): ...
class IdleInTransactionSessionTimeoutError(InvalidTransactionStateError): ...
class InvalidSQLStatementNameError(_base.PostgresError): ...
class TriggeredDataChangeViolationError(_base.PostgresError): ...
class InvalidAuthorizationSpecificationError(_base.PostgresError): ...
class InvalidPasswordError(InvalidAuthorizationSpecificationError): ...
class DependentPrivilegeDescriptorsStillExistError(_base.PostgresError): ...
class DependentObjectsStillExistError(DependentPrivilegeDescriptorsStillExistError): ...
class InvalidTransactionTerminationError(_base.PostgresError): ...
class SQLRoutineError(_base.PostgresError): ...
class FunctionExecutedNoReturnStatementError(SQLRoutineError): ...
class ModifyingSQLDataNotPermittedError(SQLRoutineError): ...
class ProhibitedSQLStatementAttemptedError(SQLRoutineError): ...
class ReadingSQLDataNotPermittedError(SQLRoutineError): ...
class InvalidCursorNameError(_base.PostgresError): ...
class ExternalRoutineError(_base.PostgresError): ...
class ContainingSQLNotPermittedError(ExternalRoutineError): ...
class ModifyingExternalRoutineSQLDataNotPermittedError(ExternalRoutineError): ...
class ProhibitedExternalRoutineSQLStatementAttemptedError(ExternalRoutineError): ...
class ReadingExternalRoutineSQLDataNotPermittedError(ExternalRoutineError): ...
class ExternalRoutineInvocationError(_base.PostgresError): ...
class InvalidSqlstateReturnedError(ExternalRoutineInvocationError): ...
class NullValueInExternalRoutineNotAllowedError(ExternalRoutineInvocationError): ...
class TriggerProtocolViolatedError(ExternalRoutineInvocationError): ...
class SrfProtocolViolatedError(ExternalRoutineInvocationError): ...
class EventTriggerProtocolViolatedError(ExternalRoutineInvocationError): ...
class SavepointError(_base.PostgresError): ...
class InvalidSavepointSpecificationError(SavepointError): ...
class InvalidCatalogNameError(_base.PostgresError): ...
class InvalidSchemaNameError(_base.PostgresError): ...
class TransactionRollbackError(_base.PostgresError): ...
class TransactionIntegrityConstraintViolationError(TransactionRollbackError): ...
class SerializationError(TransactionRollbackError): ...
class StatementCompletionUnknownError(TransactionRollbackError): ...
class DeadlockDetectedError(TransactionRollbackError): ...
class SyntaxOrAccessError(_base.PostgresError): ...
class PostgresSyntaxError(SyntaxOrAccessError): ...
class InsufficientPrivilegeError(SyntaxOrAccessError): ...
class CannotCoerceError(SyntaxOrAccessError): ...
class GroupingError(SyntaxOrAccessError): ...
class WindowingError(SyntaxOrAccessError): ...
class InvalidRecursionError(SyntaxOrAccessError): ...
class InvalidForeignKeyError(SyntaxOrAccessError): ...
class InvalidNameError(SyntaxOrAccessError): ...
class NameTooLongError(SyntaxOrAccessError): ...
class ReservedNameError(SyntaxOrAccessError): ...
class DatatypeMismatchError(SyntaxOrAccessError): ...
class IndeterminateDatatypeError(SyntaxOrAccessError): ...
class CollationMismatchError(SyntaxOrAccessError): ...
class IndeterminateCollationError(SyntaxOrAccessError): ...
class WrongObjectTypeError(SyntaxOrAccessError): ...
class GeneratedAlwaysError(SyntaxOrAccessError): ...
class UndefinedColumnError(SyntaxOrAccessError): ...
class UndefinedFunctionError(SyntaxOrAccessError): ...
class UndefinedTableError(SyntaxOrAccessError): ...
class UndefinedParameterError(SyntaxOrAccessError): ...
class UndefinedObjectError(SyntaxOrAccessError): ...
class DuplicateColumnError(SyntaxOrAccessError): ...
class DuplicateCursorError(SyntaxOrAccessError): ...
class DuplicateDatabaseError(SyntaxOrAccessError): ...
class DuplicateFunctionError(SyntaxOrAccessError): ...
class DuplicatePreparedStatementError(SyntaxOrAccessError): ...
class DuplicateSchemaError(SyntaxOrAccessError): ...
class DuplicateTableError(SyntaxOrAccessError): ...
class DuplicateAliasError(SyntaxOrAccessError): ...
class DuplicateObjectError(SyntaxOrAccessError): ...
class AmbiguousColumnError(SyntaxOrAccessError): ...
class AmbiguousFunctionError(SyntaxOrAccessError): ...
class AmbiguousParameterError(SyntaxOrAccessError): ...
class AmbiguousAliasError(SyntaxOrAccessError): ...
class InvalidColumnReferenceError(SyntaxOrAccessError): ...
class InvalidColumnDefinitionError(SyntaxOrAccessError): ...
class InvalidCursorDefinitionError(SyntaxOrAccessError): ...
class InvalidDatabaseDefinitionError(SyntaxOrAccessError): ...
class InvalidFunctionDefinitionError(SyntaxOrAccessError): ...
class InvalidPreparedStatementDefinitionError(SyntaxOrAccessError): ...
class InvalidSchemaDefinitionError(SyntaxOrAccessError): ...
class InvalidTableDefinitionError(SyntaxOrAccessError): ...
class InvalidObjectDefinitionError(SyntaxOrAccessError): ...
class WithCheckOptionViolationError(_base.PostgresError): ...
class InsufficientResourcesError(_base.PostgresError): ...
class DiskFullError(InsufficientResourcesError): ...
class OutOfMemoryError(InsufficientResourcesError): ...
class TooManyConnectionsError(InsufficientResourcesError): ...
class ConfigurationLimitExceededError(InsufficientResourcesError): ...
class ProgramLimitExceededError(_base.PostgresError): ...
class StatementTooComplexError(ProgramLimitExceededError): ...
class TooManyColumnsError(ProgramLimitExceededError): ...
class TooManyArgumentsError(ProgramLimitExceededError): ...
class ObjectNotInPrerequisiteStateError(_base.PostgresError): ...
class ObjectInUseError(ObjectNotInPrerequisiteStateError): ...
class CantChangeRuntimeParamError(ObjectNotInPrerequisiteStateError): ...
class LockNotAvailableError(ObjectNotInPrerequisiteStateError): ...
class UnsafeNewEnumValueUsageError(ObjectNotInPrerequisiteStateError): ...
class OperatorInterventionError(_base.PostgresError): ...
class QueryCanceledError(OperatorInterventionError): ...
class AdminShutdownError(OperatorInterventionError): ...
class CrashShutdownError(OperatorInterventionError): ...
class CannotConnectNowError(OperatorInterventionError): ...
class DatabaseDroppedError(OperatorInterventionError): ...
class PostgresSystemError(_base.PostgresError): ...
class PostgresIOError(PostgresSystemError): ...
class UndefinedFileError(PostgresSystemError): ...
class DuplicateFileError(PostgresSystemError): ...
class SnapshotTooOldError(_base.PostgresError): ...
class ConfigFileError(_base.PostgresError): ...
class LockFileExistsError(ConfigFileError): ...
class FDWError(_base.PostgresError): ...
class FDWColumnNameNotFoundError(FDWError): ...
class FDWDynamicParameterValueNeededError(FDWError): ...
class FDWFunctionSequenceError(FDWError): ...
class FDWInconsistentDescriptorInformationError(FDWError): ...
class FDWInvalidAttributeValueError(FDWError): ...
class FDWInvalidColumnNameError(FDWError): ...
class FDWInvalidColumnNumberError(FDWError): ...
class FDWInvalidDataTypeError(FDWError): ...
class FDWInvalidDataTypeDescriptorsError(FDWError): ...
class FDWInvalidDescriptorFieldIdentifierError(FDWError): ...
class FDWInvalidHandleError(FDWError): ...
class FDWInvalidOptionIndexError(FDWError): ...
class FDWInvalidOptionNameError(FDWError): ...
class FDWInvalidStringLengthOrBufferLengthError(FDWError): ...
class FDWInvalidStringFormatError(FDWError): ...
class FDWInvalidUseOfNullPointerError(FDWError): ...
class FDWTooManyHandlesError(FDWError): ...
class FDWOutOfMemoryError(FDWError): ...
class FDWNoSchemasError(FDWError): ...
class FDWOptionNameNotFoundError(FDWError): ...
class FDWReplyHandleError(FDWError): ...
class FDWSchemaNotFoundError(FDWError): ...
class FDWTableNotFoundError(FDWError): ...
class FDWUnableToCreateExecutionError(FDWError): ...
class FDWUnableToCreateReplyError(FDWError): ...
class FDWUnableToEstablishConnectionError(FDWError): ...
class PLPGSQLError(_base.PostgresError): ...
class RaiseError(PLPGSQLError): ...
class NoDataFoundError(PLPGSQLError): ...
class TooManyRowsError(PLPGSQLError): ...
class AssertError(PLPGSQLError): ...
class InternalServerError(_base.PostgresError): ...
class DataCorruptedError(InternalServerError): ...
class IndexCorruptedError(InternalServerError): ...



import codecs
import typing
import uuid

class CodecContext:
    def get_text_codec(self) -> codecs.CodecInfo: ...

class ReadBuffer: ...
class WriteBuffer: ...
class BufferError(Exception): ...

class UUID(uuid.UUID):
    def __init__(self, inp: typing.AnyStr) -> None: ...



import builtins
from typing import (
    Any,
    Hashable,
    Iterable,
    Iterator,
    Optional,
    Sequence,
    Sized,
    SupportsFloat,
    Text,
    Tuple,
    Type,
    TypeVar,
    Union,
    overload,
)
from typing_extensions import Literal, SupportsIndex

_BitString = TypeVar('_BitString', bound=BitString)
_BitOrderType = Literal['big', 'little']

class BitString:
    def __init__(self, bitstring: Optional[builtins.bytes] = ...) -> None: ...
    @classmethod
    def frombytes(
        cls: Type[_BitString],
        bytes_: Optional[builtins.bytes] = ...,
        bitlength: Optional[int] = ...,
    ) -> _BitString: ...
    @property
    def bytes(self) -> builtins.bytes: ...
    def as_string(self) -> str: ...
    def to_int(self, bitorder: _BitOrderType = ..., *, signed: bool = ...) -> int: ...
    @classmethod
    def from_int(
        cls: Type[_BitString],
        x: int,
        length: int,
        bitorder: _BitOrderType = ...,
        *,
        signed: bool = ...,
    ) -> _BitString: ...
    def __eq__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...
    def __getitem__(self, i: int) -> int: ...
    def __len__(self) -> int: ...

class Point(Tuple[float, float]):
    def __init__(
        self,
        x: Union[
            SupportsFloat,
            SupportsIndex,
            Text,
            builtins.bytes,
            builtins.bytearray,
        ],
        y: Union[
            SupportsFloat,
            SupportsIndex,
            Text,
            builtins.bytes,
            builtins.bytearray,
        ],
    ) -> None: ...
    @property
    def x(self) -> float: ...
    @property
    def y(self) -> float: ...

class Box(Tuple[Point, Point]):
    def __init__(self, high: Sequence[float], low: Sequence[float]) -> None: ...
    @property
    def high(self) -> Point: ...
    @property
    def low(self) -> Point: ...

class Line(Tuple[float, float, float]):
    def __init__(self, A: float, B: float, C: float) -> None: ...
    @property
    def A(self) -> float: ...
    @property
    def B(self) -> float: ...
    @property
    def C(self) -> float: ...

class LineSegment(Tuple[Point, Point]):
    def __init__(self, p1: Sequence[float], p2: Sequence[float]) -> None: ...
    @property
    def p1(self) -> Point: ...
    @property
    def p2(self) -> Point: ...

class Path:
    points: Tuple[Point, ...]
    def __init__(self, *points: Sequence[float], is_closed: bool = ...) -> None: ...
    @property
    def is_closed(self) -> bool: ...
    def __eq__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...
    def __iter__(self) -> Iterator[Point]: ...
    def __len__(self) -> int: ...
    @overload
    def __getitem__(self, i: int) -> Point: ...
    @overload
    def __getitem__(self, i: slice) -> Tuple[Point, ...]: ...
    @overload
    def __getitem__(self, i: Union[int, slice]) -> Union[Point, Tuple[Point, ...]]: ...
    def __contains__(self, point: object) -> bool: ...

class Polygon(Path):
    def __init__(self, *points: Sequence[float]) -> None: ...

class Circle(Tuple[Point, float]):
    def __init__(self, center: Point, radius: float) -> None: ...
    @property
    def center(self) -> Point: ...
    @property
    def radius(self) -> float: ...



import socket
from asyncio import AbstractEventLoop, Event, Future, Task
from threading import Thread
from typing import Any, Dict, Optional

from asyncpg import cluster as cluster

class StopServer(Exception): ...

class TCPFuzzingProxy:
    listening_addr: str = ...
    listening_port: Optional[int] = ...
    backend_host: str = ...
    backend_port: int = ...
    settings: Dict[str, Any] = ...
    loop: Optional[AbstractEventLoop] = ...
    connectivity: Optional[Event] = ...
    connectivity_loss: Optional[Event] = ...
    stop_event: Optional[Event] = ...
    connections: Dict[Connection, Task[Any]] = ...
    sock: Optional[socket.socket] = ...
    listen_task: Optional[Future[Any]] = ...
    def __init__(
        self,
        *,
        listening_addr: str = ...,
        listening_port: Optional[int] = ...,
        backend_host: str,
        backend_port: int,
        settings: Optional[Dict[str, Any]] = ...,
    ) -> None: ...
    thread: Optional[Thread] = ...
    def start(self) -> None: ...
    def stop(self) -> None: ...
    async def listen(self) -> None: ...
    def trigger_connectivity_loss(self) -> None: ...
    def restore_connectivity(self) -> None: ...
    def reset(self) -> None: ...
    def close_all_connections(self) -> None: ...

class Connection:
    client_sock: socket.socket = ...
    backend_sock: socket.socket = ...
    proxy: TCPFuzzingProxy = ...
    loop: AbstractEventLoop = ...
    connectivity: Event = ...
    connectivity_loss: Event = ...
    proxy_to_backend_task: Optional[Future[Any]] = ...
    proxy_from_backend_task: Optional[Future[Any]] = ...
    is_closed: bool = ...
    def __init__(
        self,
        client_sock: socket.socket,
        backend_sock: socket.socket,
        proxy: TCPFuzzingProxy,
    ) -> None: ...
    def close(self) -> None: ...
    async def handle(self) -> None: ...
    async def proxy_to_backend(self) -> None: ...
    async def proxy_from_backend(self) -> None: ...



import asyncio
import asyncio.protocols
import hmac
from codecs import CodecInfo
from hashlib import md5, sha256
from typing import (
    Any,
    Callable,
    ClassVar,
    Dict,
    Generic,
    Iterable,
    Iterator,
    List,
    NewType,
    Optional,
    Set,
    Text,
    Tuple,
)
from typing import Type as _TypingType
from typing import TypeVar, Union, overload
from typing_extensions import Final, Literal

import asyncpg.pgproto.pgproto

from ..connect_utils import _ConnectionParameters
from ..pgproto.pgproto import WriteBuffer
from ..types import Attribute, Type

_Record = TypeVar('_Record', bound=Record)
_OtherRecord = TypeVar('_OtherRecord', bound=Record)
_PreparedStatementState = TypeVar(
    '_PreparedStatementState', bound=PreparedStatementState[Any]
)
_T = TypeVar('_T')

_NoTimeoutType = NewType('_NoTimeoutType', object)
_TimeoutType = Union[float, None, _NoTimeoutType]

BUILTIN_TYPE_NAME_MAP: Final[Dict[str, int]]
BUILTIN_TYPE_OID_MAP: Final[Dict[int, str]]
NO_TIMEOUT: Final[_NoTimeoutType]
hashlib_md5 = md5

class ConnectionSettings(asyncpg.pgproto.pgproto.CodecContext):
    __pyx_vtable__: Any = ...
    def __init__(self, conn_key: Any) -> None: ...
    def add_python_codec(
        self,
        typeoid: int,
        typename: str,
        typeschema: str,
        typekind: str,
        encoder: Callable[[Any], Any],
        decoder: Callable[[Any], Any],
        format: Any,
    ) -> Any: ...
    def clear_type_cache(self) -> None: ...
    def get_data_codec(self, oid: int, format: Any = ...) -> Any: ...
    def get_text_codec(self) -> CodecInfo: ...
    def register_data_types(self, types: Iterable[Any]) -> None: ...
    def remove_python_codec(
        self, typeoid: int, typename: str, typeschema: str
    ) -> None: ...
    def set_builtin_type_codec(
        self,
        typeoid: int,
        typename: str,
        typeschema: str,
        typekind: str,
        alias_to: str,
        format: Any = ...,
    ) -> Any: ...
    def __getattr__(self, name: str) -> Any: ...
    def __reduce__(self) -> Any: ...

class PreparedStatementState(Generic[_Record]):
    closed: bool = ...
    name: str = ...
    query: str = ...
    refs: int = ...
    record_class: _TypingType[_Record] = ...
    ignore_custom_codec: bool = ...
    __pyx_vtable__: Any = ...
    def __init__(
        self,
        name: str,
        query: str,
        protocol: BaseProtocol[Any],
        record_class: _TypingType[_Record],
        ignore_custom_codec: bool,
    ) -> None: ...
    def _get_parameters(self) -> Tuple[Type, ...]: ...
    def _get_attributes(self) -> Tuple[Attribute, ...]: ...
    def _init_types(self) -> Set[int]: ...
    def _init_codecs(self) -> None: ...
    def attach(self) -> None: ...
    def detach(self) -> None: ...
    def mark_closed(self) -> None: ...
    def __reduce__(self) -> Any: ...

class CoreProtocol:
    backend_pid: Any = ...
    backend_secret: Any = ...
    __pyx_vtable__: Any = ...
    def __init__(self, con_params: _ConnectionParameters) -> None: ...
    def is_in_transaction(self) -> bool: ...

class BaseProtocol(CoreProtocol, Generic[_Record]):
    queries_count: Any = ...
    is_ssl: bool = ...
    __pyx_vtable__: Any = ...
    def __init__(
        self,
        addr: Any,
        connected_fut: Any,
        con_params: _ConnectionParameters,
        record_class: _TypingType[_Record],
        loop: Any,
    ) -> None: ...
    def set_connection(self, connection: Any) -> None: ...
    def get_server_pid(self, *args: Any, **kwargs: Any) -> int: ...
    def get_settings(self, *args: Any, **kwargs: Any) -> ConnectionSettings: ...
    def get_record_class(self) -> _TypingType[_Record]: ...
    def abort(self) -> None: ...
    async def bind(
        self,
        state: PreparedStatementState[_OtherRecord],
        args: Any,
        portal_name: str,
        timeout: _TimeoutType,
    ) -> Any: ...
    @overload
    async def bind_execute(
        self,
        state: PreparedStatementState[_OtherRecord],
        args: Any,
        portal_name: str,
        limit: int,
        return_extra: Literal[False],
        timeout: _TimeoutType,
    ) -> List[_OtherRecord]: ...
    @overload
    async def bind_execute(
        self,
        state: PreparedStatementState[_OtherRecord],
        args: Any,
        portal_name: str,
        limit: int,
        return_extra: Literal[True],
        timeout: _TimeoutType,
    ) -> Tuple[List[_OtherRecord], bytes, bool]: ...
    @overload
    async def bind_execute(
        self,
        state: PreparedStatementState[_OtherRecord],
        args: Any,
        portal_name: str,
        limit: int,
        return_extra: bool,
        timeout: _TimeoutType,
    ) -> Union[List[_OtherRecord], Tuple[List[_OtherRecord], bytes, bool]]: ...
    async def bind_execute_many(
        self,
        state: PreparedStatementState[_OtherRecord],
        args: Any,
        portal_name: str,
        timeout: _TimeoutType,
    ) -> None: ...
    async def close(self, timeout: _TimeoutType) -> None: ...
    def _get_timeout(self, timeout: _TimeoutType) -> Optional[float]: ...
    def _is_cancelling(self) -> bool: ...
    async def _wait_for_cancellation(self) -> None: ...
    async def close_statement(
        self, state: PreparedStatementState[_OtherRecord], timeout: _TimeoutType
    ) -> Any: ...
    async def copy_in(self, *args: Any, **kwargs: Any) -> str: ...
    async def copy_out(self, *args: Any, **kwargs: Any) -> str: ...
    async def execute(self, *args: Any, **kwargs: Any) -> Any: ...
    def is_closed(self, *args: Any, **kwargs: Any) -> Any: ...
    def is_connected(self, *args: Any, **kwargs: Any) -> Any: ...
    def is_in_transaction(self, *args: Any, **kwargs: Any) -> bool: ...
    def data_received(self, data: Any) -> None: ...
    def connection_made(self, transport: Any) -> None: ...
    def connection_lost(self, exc: Optional[Exception]) -> None: ...
    def pause_writing(self, *args: Any, **kwargs: Any) -> Any: ...
    @overload
    async def prepare(
        self,
        stmt_name: str,
        query: str,
        timeout: Optional[float] = ...,
        *,
        state: _PreparedStatementState,
        ignore_custom_codec: bool = ...,
        record_class: None,
    ) -> _PreparedStatementState: ...
    @overload
    async def prepare(
        self,
        stmt_name: str,
        query: str,
        timeout: Optional[float] = ...,
        *,
        state: None = ...,
        ignore_custom_codec: bool = ...,
        record_class: _TypingType[_OtherRecord],
    ) -> PreparedStatementState[_OtherRecord]: ...
    @overload
    async def prepare(
        self,
        stmt_name: str,
        query: str,
        timeout: Optional[float] = ...,
        *,
        state: Optional[_PreparedStatementState] = ...,
        ignore_custom_codec: bool = ...,
        record_class: Optional[_TypingType[_OtherRecord]],
    ) -> Union[_PreparedStatementState, _TypingType[_OtherRecord]]: ...
    async def query(self, *args: Any, **kwargs: Any) -> str: ...
    def resume_writing(self, *args: Any, **kwargs: Any) -> Any: ...
    def __reduce__(self) -> Any: ...

class Codec:
    __pyx_vtable__: Any = ...
    def __init__(
        self,
        name: str,
        schema: str,
        kind: str,
        type: int,
        format: int,
        xformat: int,
        c_encoder: Any,
    ) -> None: ...
    def __reduce__(self) -> Any: ...

class DataCodecConfig:
    __pyx_vtable__: Any = ...
    def __init__(self, cache_key: Any) -> None: ...
    def add_python_codec(
        self,
        typeoid: int,
        typename: str,
        typeschema: str,
        typekind: str,
        encoder: Callable[[ConnectionSettings, WriteBuffer, object], object],
        decoder: Callable[..., object],
        format: Any,
        xformat: Any,
    ) -> Any: ...
    def add_types(self, types: Iterable[Any]) -> Any: ...
    def clear_type_cache(self) -> None: ...
    def declare_fallback_codec(self, oid: int, name: str, schema: str) -> Codec: ...
    def remove_python_codec(
        self, typeoid: int, typename: str, typeschema: str
    ) -> Any: ...
    def set_builtin_type_codec(
        self,
        typeoid: int,
        typename: str,
        typeschema: str,
        typekind: str,
        alias_to: str,
        format: Any = ...,
    ) -> Any: ...
    def __reduce__(self) -> Any: ...

class Protocol(BaseProtocol[_Record], asyncio.protocols.Protocol): ...

class Record:
    @overload
    def get(self, key: str) -> Optional[Any]: ...
    @overload
    def get(self, key: str, default: _T) -> Union[Any, _T]: ...
    def items(self) -> Iterator[Tuple[str, Any]]: ...
    def keys(self) -> Iterator[str]: ...
    def values(self) -> Iterator[Any]: ...
    @overload
    def __getitem__(self, index: str) -> Any: ...
    @overload
    def __getitem__(self, index: int) -> Any: ...
    @overload
    def __getitem__(self, index: slice) -> Tuple[Any, ...]: ...
    def __iter__(self) -> Iterator[Any]: ...
    def __contains__(self, x: object) -> bool: ...
    def __len__(self) -> int: ...

class Timer:
    def __init__(self, budget: Optional[float]) -> None: ...
    def __enter__(self) -> None: ...
    def __exit__(self, et: Any, e: Any, tb: Any) -> None: ...
    def get_remaining_budget(self) -> float: ...
    def has_budget_greater_than(self, amount: float) -> bool: ...

class SCRAMAuthentication:
    AUTHENTICATION_METHODS: ClassVar[List[str]]
    DEFAULT_CLIENT_NONCE_BYTES: ClassVar[int]
    DIGEST = sha256
    REQUIREMENTS_CLIENT_FINAL_MESSAGE: ClassVar[List[str]]
    REQUIREMENTS_CLIENT_PROOF: ClassVar[List[str]]
    SASLPREP_PROHIBITED: ClassVar[Tuple[Callable[[Text], bool], ...]]
    authentication_method: bytes
    authorization_message: Optional[bytes]
    client_channel_binding: bytes
    client_first_message_bare: Optional[bytes]
    client_nonce: Optional[bytes]
    client_proof: Optional[bytes]
    password_salt: Optional[bytes]
    password_iterations: int
    server_first_message: Optional[bytes]
    server_key: Optional[hmac.HMAC]
    server_nonce: Optional[bytes]



from typing import Any, Coroutine, Generic, Iterable, List, Optional, Tuple, TypeVar
from typing_extensions import Protocol

from . import connresource, cursor
from .connection import Connection
from .protocol import Record
from .protocol.protocol import BaseProtocol, PreparedStatementState
from .types import Attribute, Type

_Record = TypeVar('_Record', bound=Record)
_T_co = TypeVar('_T_co', covariant=True)

class _Executor(Protocol[_T_co]):
    def __call__(self, __protocol: BaseProtocol[Any]) -> Coroutine[Any, Any, _T_co]: ...

class PreparedStatement(connresource.ConnectionResource, Generic[_Record]):
    def __init__(
        self,
        connection: Connection[Any],
        query: str,
        state: PreparedStatementState[_Record],
    ) -> None: ...
    def get_query(self) -> str: ...
    def get_statusmsg(self) -> Optional[str]: ...
    def get_parameters(self) -> Tuple[Type, ...]: ...
    def get_attributes(self) -> Tuple[Attribute, ...]: ...
    def cursor(
        self, *args: Any, prefetch: Optional[int] = ..., timeout: Optional[float] = ...
    ) -> cursor.CursorFactory[_Record]: ...
    async def explain(self, *args: Any, analyze: bool = ...) -> Any: ...
    async def fetch(
        self, *args: Any, timeout: Optional[float] = ...
    ) -> List[_Record]: ...
    async def fetchval(
        self, *args: Any, column: int = ..., timeout: Optional[float] = ...
    ) -> Any: ...
    async def fetchrow(
        self, *args: Any, timeout: Optional[float] = ...
    ) -> Optional[_Record]: ...
    async def executemany(
        self,
        args: Iterable[Any],
        *,
        timeout: Optional[float] = ...,
    ) -> None: ...
    def __del__(self) -> None: ...



from asyncio import AbstractEventLoop
from ssl import SSLContext
from typing import (
    IO,
    Any,
    AnyStr,
    AsyncIterable,
    BinaryIO,
    Callable,
    Coroutine,
    Dict,
    Generic,
    Iterable,
    List,
    NamedTuple,
    Optional,
    Sequence,
    Tuple,
    Type,
    TypeVar,
    Union,
    overload,
)
from typing_extensions import Literal, Protocol

from _typeshed import StrOrBytesPath

from . import connect_utils
from .cursor import CursorFactory
from .exceptions import PostgresLogMessage
from .pool import PoolConnectionProxy
from .prepared_stmt import PreparedStatement
from .protocol import Record
from .protocol.protocol import BaseProtocol, ConnectionSettings, PreparedStatementState
from .transaction import Transaction, _IsolationLevels
from .types import ServerVersion

_Connection = TypeVar('_Connection', bound='Connection[Any]')
_Writer = Callable[[bytes], Coroutine[Any, Any, None]]
_Record = TypeVar('_Record', bound=Record)
_OtherRecord = TypeVar('_OtherRecord', bound=Record)
_RecordsType = List[_Record]
_RecordsExtraType = Tuple[_RecordsType[_Record], bytes, bool]
_AnyCallable = Callable[..., Any]

_OutputType = Union[StrOrBytesPath, BinaryIO, _Writer]
_SourceType = Union[StrOrBytesPath, BinaryIO, AsyncIterable[bytes]]

_CopyFormat = Literal['text', 'csv', 'binary']
_PasswordType = Union[str, Callable[[], str], Callable[[], Coroutine[Any, Any, str]]]

class _Listener(Protocol):
    def __call__(
        self,
        __con_ref: Union[Connection[Any], PoolConnectionProxy[Any]],
        __pid: int,
        __channel: str,
        __payload: Any,
    ) -> None: ...

class _LogListener(Protocol):
    def __call__(
        self,
        __con_ref: Union[Connection[Any], PoolConnectionProxy[Any]],
        __message: PostgresLogMessage,
    ) -> None: ...

class _TerminationListener(Protocol):
    def __call__(
        self, __con_ref: Union[Connection[Any], PoolConnectionProxy[Any]]
    ) -> None: ...

class _OnRemove(Protocol[_Record]):
    def __call__(self, __statement: PreparedStatementState[_Record]) -> None: ...

class _Executor(Protocol[_Record]):
    def __call__(
        self,
        __statement: PreparedStatementState[_Record],
        __timeout: Optional[float],
    ) -> Any: ...

class ConnectionMeta(type):
    def __instancecheck__(cls, instance: Any) -> Any: ...

class Connection(Generic[_Record], metaclass=ConnectionMeta):
    def __init__(
        self,
        protocol: BaseProtocol[_Record],
        transport: Any,
        loop: AbstractEventLoop,
        addr: Union[Tuple[str, int], str],
        config: connect_utils._ClientConfiguration,
        params: connect_utils._ConnectionParameters,
    ) -> None: ...
    def __del__(self) -> None: ...
    async def add_listener(self, channel: str, callback: _Listener) -> None: ...
    async def remove_listener(self, channel: str, callback: _Listener) -> None: ...
    def add_log_listener(self, callback: _LogListener) -> None: ...
    def remove_log_listener(self, callback: _LogListener) -> None: ...
    def add_termination_listener(self, callback: _TerminationListener) -> None: ...
    def remove_termination_listener(self, callback: _TerminationListener) -> None: ...
    def get_server_pid(self) -> int: ...
    def get_server_version(self) -> ServerVersion: ...
    def get_settings(self) -> ConnectionSettings: ...
    def transaction(
        self,
        *,
        isolation: Optional[_IsolationLevels] = ...,
        readonly: bool = ...,
        deferrable: bool = ...,
    ) -> Transaction: ...
    def is_in_transaction(self) -> bool: ...
    async def execute(
        self, query: str, *args: Any, timeout: Optional[float] = ...
    ) -> str: ...
    async def executemany(
        self,
        command: str,
        args: Iterable[Sequence[Any]],
        *,
        timeout: Optional[float] = ...,
    ) -> None: ...
    @overload
    def cursor(
        self,
        query: str,
        *args: Any,
        prefetch: Optional[int] = ...,
        timeout: Optional[float] = ...,
        record_class: None = ...,
    ) -> CursorFactory[_Record]: ...
    @overload
    def cursor(
        self,
        query: str,
        *args: Any,
        prefetch: Optional[int] = ...,
        timeout: Optional[float] = ...,
        record_class: Type[_OtherRecord],
    ) -> CursorFactory[_OtherRecord]: ...
    @overload
    def cursor(
        self,
        query: str,
        *args: Any,
        prefetch: Optional[int] = ...,
        timeout: Optional[float] = ...,
        record_class: Optional[Type[_OtherRecord]],
    ) -> Union[CursorFactory[_Record], CursorFactory[_OtherRecord]]: ...
    @overload
    def cursor(
        self,
        query: str,
        *args: Any,
        prefetch: Optional[int] = ...,
        timeout: Optional[float] = ...,
        record_class: Optional[Type[_OtherRecord]] = ...,
    ) -> Union[CursorFactory[_OtherRecord], CursorFactory[_Record]]: ...
    @overload
    async def prepare(
        self,
        query: str,
        *,
        timeout: Optional[float] = ...,
        record_class: None = ...,
    ) -> PreparedStatement[_Record]: ...
    @overload
    async def prepare(
        self,
        query: str,
        *,
        timeout: Optional[float] = ...,
        record_class: Type[_OtherRecord],
    ) -> PreparedStatement[_OtherRecord]: ...
    @overload
    async def prepare(
        self,
        query: str,
        *,
        timeout: Optional[float] = ...,
        record_class: Optional[Type[_OtherRecord]],
    ) -> Union[PreparedStatement[_Record], PreparedStatement[_OtherRecord]]: ...
    @overload
    async def prepare(
        self,
        query: str,
        *,
        timeout: Optional[float] = ...,
        record_class: Optional[Type[_OtherRecord]] = ...,
    ) -> Union[PreparedStatement[_Record], PreparedStatement[_OtherRecord]]: ...
    @overload
    async def fetch(
        self,
        query: str,
        *args: Any,
        timeout: Optional[float] = ...,
        record_class: None = ...,
    ) -> List[_Record]: ...
    @overload
    async def fetch(
        self,
        query: str,
        *args: Any,
        timeout: Optional[float] = ...,
        record_class: Type[_OtherRecord],
    ) -> List[_OtherRecord]: ...
    @overload
    async def fetch(
        self,
        query: str,
        *args: Any,
        timeout: Optional[float] = ...,
        record_class: Optional[Type[_OtherRecord]],
    ) -> Union[List[_Record], List[_OtherRecord]]: ...
    @overload
    async def fetch(
        self,
        query: str,
        *args: Any,
        timeout: Optional[float] = ...,
        record_class: Optional[Type[_OtherRecord]] = ...,
    ) -> Union[List[_Record], List[_OtherRecord]]: ...
    async def fetchval(
        self, query: str, *args: Any, column: int = ..., timeout: Optional[float] = ...
    ) -> Optional[Any]: ...
    @overload
    async def fetchrow(
        self,
        query: str,
        *args: Any,
        timeout: Optional[float] = ...,
        record_class: None = ...,
    ) -> Optional[_Record]: ...
    @overload
    async def fetchrow(
        self,
        query: str,
        *args: Any,
        timeout: Optional[float] = ...,
        record_class: Type[_OtherRecord],
    ) -> Optional[_OtherRecord]: ...
    @overload
    async def fetchrow(
        self,
        query: str,
        *args: Any,
        timeout: Optional[float] = ...,
        record_class: Optional[Type[_OtherRecord]],
    ) -> Union[Optional[_Record], Optional[_OtherRecord]]: ...
    @overload
    async def fetchrow(
        self,
        query: str,
        *args: Any,
        timeout: Optional[float] = ...,
        record_class: Optional[Type[_OtherRecord]] = ...,
    ) -> Union[None, _Record, _OtherRecord]: ...
    async def copy_from_table(
        self,
        table_name: str,
        output: _OutputType,
        *,
        columns: Optional[Iterable[str]] = ...,
        schema_name: Optional[str] = ...,
        timeout: Optional[float] = ...,
        format: Optional[_CopyFormat] = ...,
        oids: Optional[int] = ...,
        delimiter: Optional[str] = ...,
        null: Optional[str] = ...,
        header: Optional[bool] = ...,
        quote: Optional[str] = ...,
        escape: Optional[str] = ...,
        force_quote: Optional[bool] = ...,
        encoding: Optional[str] = ...,
    ) -> str: ...
    async def copy_from_query(
        self,
        query: str,
        *args: Any,
        output: _OutputType,
        timeout: Optional[float] = ...,
        format: Optional[_CopyFormat] = ...,
        oids: Optional[int] = ...,
        delimiter: Optional[str] = ...,
        null: Optional[str] = ...,
        header: Optional[bool] = ...,
        quote: Optional[str] = ...,
        escape: Optional[str] = ...,
        force_quote: Optional[bool] = ...,
        encoding: Optional[str] = ...,
    ) -> str: ...
    async def copy_to_table(
        self,
        table_name: str,
        source: _SourceType,
        *,
        columns: Optional[Iterable[str]] = ...,
        schema_name: Optional[str] = ...,
        timeout: Optional[float] = ...,
        format: Optional[_CopyFormat] = ...,
        oids: Optional[int] = ...,
        freeze: Optional[bool] = ...,
        delimiter: Optional[str] = ...,
        null: Optional[str] = ...,
        header: Optional[bool] = ...,
        quote: Optional[str] = ...,
        escape: Optional[str] = ...,
        force_quote: Optional[bool] = ...,
        force_not_null: Optional[bool] = ...,
        force_null: Optional[bool] = ...,
        encoding: Optional[str] = ...,
    ) -> str: ...
    async def copy_records_to_table(
        self,
        table_name: str,
        records: Iterable[Sequence[Any]],
        *,
        columns: Optional[Iterable[str]] = ...,
        schema_name: Optional[str] = ...,
        timeout: Optional[float] = ...,
    ) -> str: ...
    async def set_type_codec(
        self,
        typename: str,
        *,
        schema: str = ...,
        encoder: Callable[[Any], Any],
        decoder: Callable[[Any], Any],
        format: str = ...,
    ) -> None: ...
    async def reset_type_codec(self, typename: str, *, schema: str = ...) -> None: ...
    async def set_builtin_type_codec(
        self,
        typename: str,
        *,
        schema: str = ...,
        codec_name: str,
        format: Optional[str] = ...,
    ) -> None: ...
    def is_closed(self) -> bool: ...
    async def close(self, *, timeout: Optional[float] = ...) -> None: ...
    def terminate(self) -> None: ...
    async def reset(self, *, timeout: Optional[float] = ...) -> None: ...
    async def reload_schema_state(self) -> None: ...

@overload
async def connect(
    dsn: Optional[str] = ...,
    *,
    host: Optional[connect_utils._HostType] = ...,
    port: Optional[connect_utils._PortType] = ...,
    user: Optional[str] = ...,
    password: Optional[_PasswordType] = ...,
    passfile: Optional[str] = ...,
    database: Optional[str] = ...,
    loop: Optional[AbstractEventLoop] = ...,
    timeout: float = ...,
    statement_cache_size: int = ...,
    max_cached_statement_lifetime: int = ...,
    max_cacheable_statement_size: int = ...,
    command_timeout: Optional[float] = ...,
    ssl: Optional[connect_utils._SSLType] = ...,
    connection_class: Type[Connection[_Record]] = ...,
    record_class: Type[_Record],
    server_settings: Optional[Dict[str, str]] = ...,
) -> Connection[_Record]: ...
@overload
async def connect(
    dsn: Optional[str] = ...,
    *,
    host: Optional[connect_utils._HostType] = ...,
    port: Optional[connect_utils._PortType] = ...,
    user: Optional[str] = ...,
    password: Optional[_PasswordType] = ...,
    passfile: Optional[str] = ...,
    database: Optional[str] = ...,
    loop: Optional[AbstractEventLoop] = ...,
    timeout: float = ...,
    statement_cache_size: int = ...,
    max_cached_statement_lifetime: int = ...,
    max_cacheable_statement_size: int = ...,
    command_timeout: Optional[float] = ...,
    ssl: Optional[connect_utils._SSLType] = ...,
    connection_class: Type[Connection[Record]] = ...,
    record_class: Type[Record] = ...,
    server_settings: Optional[Dict[str, str]] = ...,
) -> Connection[Record]: ...
@overload
async def connect(
    dsn: Optional[str] = ...,
    *,
    host: Optional[connect_utils._HostType] = ...,
    port: Optional[connect_utils._PortType] = ...,
    user: Optional[str] = ...,
    password: Optional[_PasswordType] = ...,
    passfile: Optional[str] = ...,
    database: Optional[str] = ...,
    loop: Optional[AbstractEventLoop] = ...,
    timeout: float = ...,
    statement_cache_size: int = ...,
    max_cached_statement_lifetime: int = ...,
    max_cacheable_statement_size: int = ...,
    command_timeout: Optional[float] = ...,
    ssl: Optional[connect_utils._SSLType] = ...,
    connection_class: Type[_Connection],
    record_class: Type[_Record] = ...,
    server_settings: Optional[Dict[str, str]] = ...,
) -> _Connection: ...

class _ConnectionProxy(Generic[_Record]): ...

class ServerCapabilities(NamedTuple):
    advisory_locks: bool
    notifications: bool
    plpgsql: bool
    sql_reset: bool
    sql_close_all: bool



from typing import (
    Any,
    AsyncIterable,
    AsyncIterator,
    Awaitable,
    Generator,
    Generic,
    List,
    Optional,
    Sequence,
    Type,
    TypeVar,
    overload,
)

from . import connresource
from .connection import Connection
from .protocol import Record
from .protocol.protocol import PreparedStatementState

_Cursor = TypeVar('_Cursor', bound=Cursor[Any])
_CursorIterator = TypeVar('_CursorIterator', bound=CursorIterator[Any])
_Record = TypeVar('_Record', bound=Record)

class CursorFactory(connresource.ConnectionResource, Generic[_Record]):
    @overload
    def __init__(
        self,
        connection: Connection[_Record],
        query: str,
        state: Optional[PreparedStatementState[_Record]],
        args: Sequence[Any],
        prefetch: Optional[int],
        timeout: Optional[float],
        record_class: None,
    ) -> None: ...
    @overload
    def __init__(
        self,
        connection: Connection[Any],
        query: str,
        state: Optional[PreparedStatementState[_Record]],
        args: Sequence[Any],
        prefetch: Optional[int],
        timeout: Optional[float],
        record_class: Type[_Record],
    ) -> None: ...
    @overload
    def __init__(
        self,
        connection: Connection[Any],
        query: str,
        state: Optional[PreparedStatementState[_Record]],
        args: Sequence[Any],
        prefetch: Optional[int],
        timeout: Optional[float],
        record_class: Optional[Type[_Record]],
    ) -> None: ...
    def __aiter__(self) -> CursorIterator[_Record]: ...
    def __await__(self) -> Generator[Any, None, Cursor[_Record]]: ...
    def __del__(self) -> None: ...

class BaseCursor(connresource.ConnectionResource, Generic[_Record]):
    @overload
    def __init__(
        self,
        connection: Connection[_Record],
        query: str,
        state: Optional[PreparedStatementState[_Record]],
        args: Sequence[Any],
        record_class: None,
    ) -> None: ...
    @overload
    def __init__(
        self,
        connection: Connection[Any],
        query: str,
        state: Optional[PreparedStatementState[_Record]],
        args: Sequence[Any],
        record_class: Type[_Record],
    ) -> None: ...
    @overload
    def __init__(
        self,
        connection: Connection[Any],
        query: str,
        state: Optional[PreparedStatementState[_Record]],
        args: Sequence[Any],
        record_class: Optional[Type[_Record]],
    ) -> None: ...
    def __del__(self) -> None: ...

class CursorIterator(BaseCursor[_Record]):
    @overload
    def __init__(
        self,
        connection: Connection[_Record],
        query: str,
        state: Optional[PreparedStatementState[_Record]],
        args: Sequence[Any],
        record_class: None,
        prefetch: int,
        timeout: Optional[float],
    ) -> None: ...
    @overload
    def __init__(
        self,
        connection: Connection[Any],
        query: str,
        state: Optional[PreparedStatementState[_Record]],
        args: Sequence[Any],
        record_class: Type[_Record],
        prefetch: int,
        timeout: Optional[float],
    ) -> None: ...
    @overload
    def __init__(
        self,
        connection: Connection[Any],
        query: str,
        state: Optional[PreparedStatementState[_Record]],
        args: Sequence[Any],
        record_class: Optional[Type[_Record]],
        prefetch: int,
        timeout: Optional[float],
    ) -> None: ...
    def __aiter__(self: _CursorIterator) -> _CursorIterator: ...
    async def __anext__(self) -> _Record: ...

class Cursor(BaseCursor[_Record]):
    async def fetch(
        self, n: int, *, timeout: Optional[float] = ...
    ) -> List[_Record]: ...
    async def fetchrow(
        self, *, timeout: Optional[float] = ...
    ) -> Optional[_Record]: ...
    async def forward(self, n: int, *, timeout: Optional[float] = ...) -> int: ...



from typing import (
    Any,
    Generic,
    Hashable,
    Iterable,
    Iterator,
    NamedTuple,
    Optional,
    Sequence,
    Sized,
    Tuple,
    TypeVar,
)

from asyncpg.pgproto.types import BitString as BitString
from asyncpg.pgproto.types import Box as Box
from asyncpg.pgproto.types import Circle as Circle
from asyncpg.pgproto.types import Line as Line
from asyncpg.pgproto.types import LineSegment as LineSegment
from asyncpg.pgproto.types import Path as Path
from asyncpg.pgproto.types import Point as Point
from asyncpg.pgproto.types import Polygon as Polygon

_T = TypeVar('_T')

class Type(NamedTuple):
    oid: int
    name: str
    kind: str
    schema: str

class Attribute(NamedTuple):
    name: str
    type: Type

class ServerVersion(NamedTuple):
    major: int
    minor: int
    micro: int
    releaselevel: str
    serial: int

class Range(Generic[_T]):
    def __init__(
        self,
        lower: Optional[_T] = ...,
        upper: Optional[_T] = ...,
        *,
        lower_inc: bool = ...,
        upper_inc: bool = ...,
        empty: bool = ...,
    ) -> None: ...
    @property
    def lower(self) -> Optional[_T]: ...
    @property
    def lower_inc(self) -> bool: ...
    @property
    def lower_inf(self) -> bool: ...
    @property
    def upper(self) -> Optional[_T]: ...
    @property
    def upper_inc(self) -> bool: ...
    @property
    def upper_inf(self) -> bool: ...
    @property
    def isempty(self) -> bool: ...
    def __bool__(self) -> bool: ...
    def __eq__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...



from .connection import Connection as Connection
from .connection import connect as connect
from .exceptions import *
from .pool import Pool as Pool
from .pool import create_pool as create_pool
from .protocol import Record as Record
from .types import *



import typing

__all__ = (
    'PostgresError',
    'FatalPostgresError',
    'UnknownPostgresError',
    'InterfaceError',
    'InterfaceWarning',
    'PostgresLogMessage',
    'InternalClientError',
    'OutdatedSchemaCacheError',
    'ProtocolError',
    'UnsupportedClientFeatureError',
)

class PostgresMessageMeta(type): ...

class PostgresMessage(metaclass=PostgresMessageMeta):
    severity: typing.Optional[str]
    severity_en: typing.Optional[str]
    sqlstate: typing.ClassVar[str]
    message: str
    detail: typing.Optional[str]
    hint: typing.Optional[str]
    position: typing.Optional[str]
    internal_position: typing.Optional[str]
    internal_query: typing.Optional[str]
    context: typing.Optional[str]
    schema_name: typing.Optional[str]
    table_name: typing.Optional[str]
    column_name: typing.Optional[str]
    data_type_name: typing.Optional[str]
    constraint_name: typing.Optional[str]
    server_source_filename: typing.Optional[str]
    server_source_line: typing.Optional[str]
    server_source_function: typing.Optional[str]
    def as_dict(self) -> typing.Dict[str, str]: ...

class PostgresError(PostgresMessage, Exception):
    def __str__(self) -> str: ...
    @classmethod
    def new(
        cls, fields: typing.Dict[str, str], query: typing.Optional[str] = ...
    ) -> PostgresMessage: ...

class FatalPostgresError(PostgresError): ...
class UnknownPostgresError(FatalPostgresError): ...

class InterfaceMessage:
    def __init__(
        self, *, detail: typing.Optional[str] = ..., hint: typing.Optional[str] = ...
    ) -> None: ...
    def with_msg(self, msg: str) -> str: ...
    def __str__(self) -> str: ...

class InterfaceError(InterfaceMessage, Exception):
    def __init__(
        self,
        msg: str,
        *,
        detail: typing.Optional[str] = ...,
        hint: typing.Optional[str] = ...,
    ) -> None: ...

class DataError(InterfaceError, ValueError): ...
class UnsupportedClientFeatureError(InterfaceError): ...

class InterfaceWarning(InterfaceMessage, UserWarning):
    def __init__(
        self,
        msg: str,
        *,
        detail: typing.Optional[str] = ...,
        hint: typing.Optional[str] = ...,
    ) -> None: ...

class InternalClientError(Exception): ...
class ProtocolError(InternalClientError): ...

class OutdatedSchemaCacheError(InternalClientError):
    def __init__(
        self,
        msg: str,
        *,
        schema: typing.Optional[str] = ...,
        data_type: typing.Optional[str] = ...,
        position: typing.Optional[str] = ...,
    ) -> None: ...

class PostgresLogMessage(PostgresMessage):
    def __str__(self) -> str: ...
    @classmethod
    def new(
        cls, fields: typing.Dict[str, str], query: typing.Optional[str] = ...
    ) -> PostgresMessage: ...



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['asyncpg-stubs']

package_data = \
{'': ['*'],
 'asyncpg-stubs': ['_testbase/*',
                   'exceptions/*',
                   'pgproto/*',
                   'protocol/*',
                   'protocol/codecs/*']}

install_requires = \
['asyncpg>=0.25.0,<0.26.0', 'mypy>=0.800', 'typing-extensions>=4.1.1,<5.0.0']

setup_kwargs = {
    'name': 'asyncpg-stubs-h1',
    'version': '0.23.2',
    'description': 'asyncpg stubs (pypi-upload copy)',
    'long_description': None,
    'author': 'Bryan Forbes',
    'author_email': 'bryan@reigndropsfall.net',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6.2,<4.0.0',
}


setup(**setup_kwargs)



import enum
from typing import Any, Dict, Optional, Set
from typing_extensions import Final, Literal

from . import connresource
from .connection import Connection

class TransactionState(enum.Enum):
    NEW: int = ...
    STARTED: int = ...
    COMMITTED: int = ...
    ROLLEDBACK: int = ...
    FAILED: int = ...

_IsolationLevels = Literal['read_committed', 'serializable', 'repeatable_read']
ISOLATION_LEVELS: Final[Set[_IsolationLevels]]
ISOLATION_LEVELS_BY_VALUE: Final[Dict[str, _IsolationLevels]]

class Transaction(connresource.ConnectionResource):
    def __init__(
        self,
        connection: Connection[Any],
        isolation: Optional[_IsolationLevels],
        readonly: bool,
        deferrable: bool,
    ) -> None: ...
    async def __aenter__(self) -> None: ...
    async def __aexit__(self, extype: Any, ex: Any, tb: Any) -> None: ...
    async def start(self) -> None: ...
    async def commit(self) -> None: ...
    async def rollback(self) -> None: ...



from asyncio import AbstractEventLoop, Future, StreamWriter, Task
from pathlib import Path
from typing import Any, AnyStr, Optional, TypeVar, overload
from typing_extensions import Final, Protocol

PY_36: Final[bool]
PY_37: Final[bool]
SYSTEM: Final[str]

_T = TypeVar('_T')
_T_co = TypeVar('_T_co', covariant=True)

class _PathLike(Protocol[_T_co]):
    def __fspath__(self) -> _T_co: ...

def aiter_compat(func: Any) -> Any: ...
@overload
def fspath(path: str) -> str: ...
@overload
def fspath(path: bytes) -> bytes: ...
@overload
def fspath(path: _PathLike[AnyStr]) -> Any: ...
def get_pg_home_directory() -> Optional[Path]: ...
def current_asyncio_task(loop: Optional[AbstractEventLoop]) -> Optional[Task[Any]]: ...
async def wait_closed(stream: StreamWriter) -> None: ...
async def wait_for(fut: Future[_T], timeout: float) -> _T: ...



import asyncio
import typing
import typing_extensions

from . import connect_utils, connection, cursor, prepared_stmt, transaction, types
from .protocol import protocol as _cprotocol

_Connection = typing.TypeVar('_Connection', bound=connection.Connection[typing.Any])
_Pool = typing.TypeVar('_Pool', bound=Pool[typing.Any])
_Record = typing.TypeVar('_Record', bound=_cprotocol.Record)
_OtherRecord = typing.TypeVar('_OtherRecord', bound=_cprotocol.Record)

class _SetupCallback(typing_extensions.Protocol[_Record]):
    async def __call__(self, __proxy: PoolConnectionProxy[_Record]) -> None: ...

class _InitCallback(typing_extensions.Protocol[_Record]):
    async def __call__(self, __con: connection.Connection[_Record]) -> None: ...

class PoolConnectionProxyMeta(type): ...

class PoolConnectionProxy(
    connection._ConnectionProxy[_Record], metaclass=PoolConnectionProxyMeta, wrap=True
):
    _holder: PoolConnectionHolder[_Record]
    async def add_listener(
        self, channel: str, callback: connection._Listener
    ) -> None: ...
    async def remove_listener(
        self, channel: str, callback: connection._Listener
    ) -> None: ...
    def add_log_listener(self, callback: connection._LogListener) -> None: ...
    def remove_log_listener(self, callback: connection._LogListener) -> None: ...
    def add_termination_listener(
        self, callback: connection._TerminationListener
    ) -> None: ...
    def remove_termination_listener(
        self, callback: connection._TerminationListener
    ) -> None: ...
    def get_server_pid(self) -> int: ...
    def get_server_version(self) -> types.ServerVersion: ...
    def get_settings(self) -> _cprotocol.ConnectionSettings: ...
    def transaction(
        self,
        *,
        isolation: typing.Optional[transaction._IsolationLevels] = ...,
        readonly: bool = ...,
        deferrable: bool = ...,
    ) -> transaction.Transaction: ...
    def is_in_transaction(self) -> bool: ...
    async def execute(
        self, query: str, *args: typing.Any, timeout: typing.Optional[float] = ...
    ) -> str: ...
    async def executemany(
        self,
        command: str,
        args: typing.Iterable[typing.Sequence[typing.Any]],
        *,
        timeout: typing.Optional[float] = ...,
    ) -> None: ...
    @typing.overload
    def cursor(
        self,
        query: str,
        *args: typing.Any,
        prefetch: typing.Optional[int] = ...,
        timeout: typing.Optional[float] = ...,
        record_class: None = ...,
    ) -> cursor.CursorFactory[_Record]: ...
    @typing.overload
    def cursor(
        self,
        query: str,
        *args: typing.Any,
        prefetch: typing.Optional[int] = ...,
        timeout: typing.Optional[float] = ...,
        record_class: typing.Type[_OtherRecord],
    ) -> cursor.CursorFactory[_OtherRecord]: ...
    @typing.overload
    def cursor(
        self,
        query: str,
        *args: typing.Any,
        prefetch: typing.Optional[int] = ...,
        timeout: typing.Optional[float] = ...,
        record_class: typing.Optional[typing.Type[_OtherRecord]],
    ) -> typing.Union[
        cursor.CursorFactory[_Record],
        cursor.CursorFactory[_OtherRecord],
    ]: ...
    @typing.overload
    async def prepare(
        self,
        query: str,
        *,
        timeout: typing.Optional[float] = ...,
        record_class: None = ...,
    ) -> prepared_stmt.PreparedStatement[_Record]: ...
    @typing.overload
    async def prepare(
        self,
        query: str,
        *,
        timeout: typing.Optional[float] = ...,
        record_class: typing.Type[_OtherRecord],
    ) -> prepared_stmt.PreparedStatement[_OtherRecord]: ...
    @typing.overload
    async def prepare(
        self,
        query: str,
        *,
        timeout: typing.Optional[float] = ...,
        record_class: typing.Optional[typing.Type[_OtherRecord]],
    ) -> typing.Union[
        prepared_stmt.PreparedStatement[_Record],
        prepared_stmt.PreparedStatement[_OtherRecord],
    ]: ...
    @typing.overload
    async def fetch(
        self,
        query: str,
        *args: typing.Any,
        timeout: typing.Optional[float] = ...,
        record_class: None = ...,
    ) -> typing.List[_Record]: ...
    @typing.overload
    async def fetch(
        self,
        query: str,
        *args: typing.Any,
        timeout: typing.Optional[float] = ...,
        record_class: typing.Type[_OtherRecord],
    ) -> typing.List[_OtherRecord]: ...
    @typing.overload
    async def fetch(
        self,
        query: str,
        *args: typing.Any,
        timeout: typing.Optional[float] = ...,
        record_class: typing.Optional[typing.Type[_OtherRecord]],
    ) -> typing.Union[typing.List[_Record], typing.List[_OtherRecord]]: ...
    async def fetchval(
        self,
        query: str,
        *args: typing.Any,
        column: int = ...,
        timeout: typing.Optional[float] = ...,
    ) -> typing.Any: ...
    @typing.overload
    async def fetchrow(
        self,
        query: str,
        *args: typing.Any,
        timeout: typing.Optional[float] = ...,
        record_class: None = ...,
    ) -> typing.Optional[_Record]: ...
    @typing.overload
    async def fetchrow(
        self,
        query: str,
        *args: typing.Any,
        timeout: typing.Optional[float] = ...,
        record_class: typing.Type[_OtherRecord],
    ) -> typing.Optional[_OtherRecord]: ...
    @typing.overload
    async def fetchrow(
        self,
        query: str,
        *args: typing.Any,
        timeout: typing.Optional[float] = ...,
        record_class: typing.Optional[typing.Type[_OtherRecord]],
    ) -> typing.Union[typing.Optional[_Record], typing.Optional[_OtherRecord]]: ...
    async def copy_from_table(
        self,
        table_name: str,
        *,
        output: connection._OutputType,
        columns: typing.Optional[typing.Iterable[str]] = ...,
        schema_name: typing.Optional[str] = ...,
        timeout: typing.Optional[float] = ...,
        format: typing.Optional[connection._CopyFormat] = ...,
        oids: typing.Optional[int] = ...,
        delimiter: typing.Optional[str] = ...,
        null: typing.Optional[str] = ...,
        header: typing.Optional[bool] = ...,
        quote: typing.Optional[str] = ...,
        escape: typing.Optional[str] = ...,
        force_quote: typing.Optional[bool] = ...,
        encoding: typing.Optional[str] = ...,
    ) -> str: ...
    async def copy_from_query(
        self,
        query: str,
        *args: typing.Any,
        output: connection._OutputType,
        timeout: typing.Optional[float] = ...,
        format: typing.Optional[connection._CopyFormat] = ...,
        oids: typing.Optional[int] = ...,
        delimiter: typing.Optional[str] = ...,
        null: typing.Optional[str] = ...,
        header: typing.Optional[bool] = ...,
        quote: typing.Optional[str] = ...,
        escape: typing.Optional[str] = ...,
        force_quote: typing.Optional[bool] = ...,
        encoding: typing.Optional[str] = ...,
    ) -> str: ...
    async def copy_to_table(
        self,
        table_name: str,
        *,
        source: connection._SourceType,
        columns: typing.Optional[typing.Iterable[str]] = ...,
        schema_name: typing.Optional[str] = ...,
        timeout: typing.Optional[float] = ...,
        format: typing.Optional[connection._CopyFormat] = ...,
        oids: typing.Optional[int] = ...,
        freeze: typing.Optional[bool] = ...,
        delimiter: typing.Optional[str] = ...,
        null: typing.Optional[str] = ...,
        header: typing.Optional[bool] = ...,
        quote: typing.Optional[str] = ...,
        escape: typing.Optional[str] = ...,
        force_quote: typing.Optional[bool] = ...,
        force_not_null: typing.Optional[bool] = ...,
        force_null: typing.Optional[bool] = ...,
        encoding: typing.Optional[str] = ...,
    ) -> str: ...
    async def copy_records_to_table(
        self,
        table_name: str,
        *,
        records: typing.Iterable[typing.Sequence[typing.Any]],
        columns: typing.Optional[typing.Iterable[str]] = ...,
        schema_name: typing.Optional[str] = ...,
        timeout: typing.Optional[float] = ...,
    ) -> str: ...
    async def set_type_codec(
        self,
        typename: str,
        *,
        schema: str = ...,
        encoder: typing.Callable[[typing.Any], typing.Any],
        decoder: typing.Callable[[typing.Any], typing.Any],
        format: str = ...,
    ) -> None: ...
    async def reset_type_codec(self, typename: str, *, schema: str = ...) -> None: ...
    async def set_builtin_type_codec(
        self,
        typename: str,
        *,
        schema: str = ...,
        codec_name: str,
        format: typing.Optional[str] = ...,
    ) -> None: ...
    def is_closed(self) -> bool: ...
    async def close(self, *, timeout: typing.Optional[float] = ...) -> None: ...
    def terminate(self) -> None: ...
    async def reset(self, *, timeout: typing.Optional[float] = ...) -> None: ...
    async def reload_schema_state(self) -> None: ...

class PoolConnectionHolder(typing.Generic[_Record]):
    _pool: Pool[_Record]
    async def connect(self) -> None: ...
    async def acquire(self) -> PoolConnectionProxy[_Record]: ...
    async def release(self, timeout: typing.Optional[float]) -> None: ...
    async def wait_until_released(self) -> None: ...
    async def close(self) -> None: ...
    def terminate(self) -> None: ...
    def _release_on_close(self) -> None: ...

class Pool(typing.Generic[_Record]):
    def set_connect_args(
        self,
        dsn: typing.Optional[str] = ...,
        *,
        host: typing.Optional[connect_utils._HostType] = ...,
        port: typing.Optional[connect_utils._PortType] = ...,
        user: typing.Optional[str] = ...,
        password: typing.Optional[connection._PasswordType] = ...,
        passfile: typing.Optional[str] = ...,
        database: typing.Optional[str] = ...,
        timeout: float = ...,
        statement_cache_size: int = ...,
        max_cached_statement_lifetime: int = ...,
        max_cacheable_statement_size: int = ...,
        command_timeout: typing.Optional[float] = ...,
        ssl: typing.Optional[connect_utils._SSLType] = ...,
        server_settings: typing.Optional[typing.Dict[str, str]] = ...,
    ) -> None: ...
    async def execute(
        self, query: str, *args: typing.Any, timeout: typing.Optional[float] = ...
    ) -> str: ...
    async def executemany(
        self,
        command: str,
        args: typing.Iterable[typing.Sequence[typing.Any]],
        *,
        timeout: typing.Optional[float] = ...,
    ) -> None: ...
    async def fetch(
        self, query: str, *args: typing.Any, timeout: typing.Optional[float] = ...
    ) -> typing.List[_cprotocol.Record]: ...
    async def fetchval(
        self,
        query: str,
        *args: typing.Any,
        column: int = ...,
        timeout: typing.Optional[float] = ...,
    ) -> typing.Any: ...
    async def fetchrow(
        self, query: str, *args: typing.Any, timeout: typing.Optional[float] = ...
    ) -> typing.Optional[_cprotocol.Record]: ...
    async def copy_from_table(
        self,
        table_name: str,
        *,
        output: connection._OutputType,
        columns: typing.Optional[typing.Iterable[str]] = ...,
        schema_name: typing.Optional[str] = ...,
        timeout: typing.Optional[float] = ...,
        format: typing.Optional[connection._CopyFormat] = ...,
        oids: typing.Optional[int] = ...,
        delimiter: typing.Optional[str] = ...,
        null: typing.Optional[str] = ...,
        header: typing.Optional[bool] = ...,
        quote: typing.Optional[str] = ...,
        escape: typing.Optional[str] = ...,
        force_quote: typing.Optional[bool] = ...,
        encoding: typing.Optional[str] = ...,
    ) -> str: ...
    async def copy_from_query(
        self,
        query: str,
        *args: typing.Any,
        output: connection._OutputType,
        timeout: typing.Optional[float] = ...,
        format: typing.Optional[connection._CopyFormat] = ...,
        oids: typing.Optional[int] = ...,
        delimiter: typing.Optional[str] = ...,
        null: typing.Optional[str] = ...,
        header: typing.Optional[bool] = ...,
        quote: typing.Optional[str] = ...,
        escape: typing.Optional[str] = ...,
        force_quote: typing.Optional[bool] = ...,
        encoding: typing.Optional[str] = ...,
    ) -> str: ...
    async def copy_to_table(
        self,
        table_name: str,
        *,
        source: connection._SourceType,
        columns: typing.Optional[typing.Iterable[str]] = ...,
        schema_name: typing.Optional[str] = ...,
        timeout: typing.Optional[float] = ...,
        format: typing.Optional[connection._CopyFormat] = ...,
        oids: typing.Optional[int] = ...,
        freeze: typing.Optional[bool] = ...,
        delimiter: typing.Optional[str] = ...,
        null: typing.Optional[str] = ...,
        header: typing.Optional[bool] = ...,
        quote: typing.Optional[str] = ...,
        escape: typing.Optional[str] = ...,
        force_quote: typing.Optional[bool] = ...,
        force_not_null: typing.Optional[bool] = ...,
        force_null: typing.Optional[bool] = ...,
        encoding: typing.Optional[str] = ...,
    ) -> str: ...
    async def copy_records_to_table(
        self,
        table_name: str,
        *,
        records: typing.Iterable[typing.Sequence[typing.Any]],
        columns: typing.Optional[typing.Iterable[str]] = ...,
        schema_name: typing.Optional[str] = ...,
        timeout: typing.Optional[float] = ...,
    ) -> str: ...
    def acquire(
        self, *, timeout: typing.Optional[float] = ...
    ) -> PoolAcquireContext[_Record]: ...
    async def release(
        self,
        connection: PoolConnectionProxy[_Record],
        *,
        timeout: typing.Optional[float] = ...,
    ) -> None: ...
    async def close(self) -> None: ...
    def terminate(self) -> None: ...
    async def expire_connections(self) -> None: ...
    def _drop_statement_cache(self) -> None: ...
    def _drop_type_cache(self) -> None: ...
    def __await__(
        self: _Pool,
    ) -> typing.Generator[typing.Any, None, typing.Optional[_Pool]]: ...
    async def __aenter__(self: _Pool) -> _Pool: ...
    async def __aexit__(self, *exc: typing.Any) -> None: ...

class PoolAcquireContext(typing.Generic[_Record]):
    async def __aenter__(self) -> PoolConnectionProxy[_Record]: ...
    async def __aexit__(self, *exc: typing.Any) -> None: ...
    def __await__(
        self,
    ) -> typing.Generator[typing.Any, None, PoolConnectionProxy[_Record]]: ...

@typing.overload
def create_pool(
    dsn: typing.Optional[str] = ...,
    *,
    min_size: int = ...,
    max_size: int = ...,
    max_queries: int = ...,
    max_inactive_connection_lifetime: float = ...,
    setup: typing.Optional[_SetupCallback[_cprotocol.Record]] = ...,
    init: typing.Optional[_InitCallback[_cprotocol.Record]] = ...,
    loop: typing.Optional[asyncio.AbstractEventLoop] = ...,
    connection_class: typing.Type[_Connection] = ...,
    record_class: typing.Optional[typing.Type[_cprotocol.Record]] = ...,
    host: typing.Optional[connect_utils._HostType] = ...,
    port: typing.Optional[connect_utils._PortType] = ...,
    user: typing.Optional[str] = ...,
    password: typing.Optional[connection._PasswordType] = ...,
    passfile: typing.Optional[str] = ...,
    database: typing.Optional[str] = ...,
    timeout: float = ...,
    statement_cache_size: int = ...,
    max_cached_statement_lifetime: int = ...,
    max_cacheable_statement_size: int = ...,
    command_timeout: typing.Optional[float] = ...,
    ssl: typing.Optional[connect_utils._SSLType] = ...,
    server_settings: typing.Optional[typing.Dict[str, str]] = ...,
) -> Pool[_cprotocol.Record]: ...
@typing.overload
def create_pool(
    dsn: typing.Optional[str] = ...,
    *,
    min_size: int = ...,
    max_size: int = ...,
    max_queries: int = ...,
    max_inactive_connection_lifetime: float = ...,
    setup: typing.Optional[_SetupCallback[_Record]] = ...,
    init: typing.Optional[_InitCallback[_Record]] = ...,
    loop: typing.Optional[asyncio.AbstractEventLoop] = ...,
    connection_class: typing.Type[_Connection] = ...,
    record_class: typing.Type[_Record],
    host: typing.Optional[connect_utils._HostType] = ...,
    port: typing.Optional[connect_utils._PortType] = ...,
    user: typing.Optional[str] = ...,
    password: typing.Optional[connection._PasswordType] = ...,
    passfile: typing.Optional[str] = ...,
    database: typing.Optional[str] = ...,
    timeout: float = ...,
    statement_cache_size: int = ...,
    max_cached_statement_lifetime: int = ...,
    max_cacheable_statement_size: int = ...,
    command_timeout: typing.Optional[float] = ...,
    ssl: typing.Optional[connect_utils._SSLType] = ...,
    server_settings: typing.Optional[typing.Dict[str, str]] = ...,
) -> Pool[_Record]: ...
